package ntapcli

// TODO: This file will be deprecated

import (
	"net/http"

	"github.com/sirupsen/logrus"

	"github.com/greenqloud/ontap-rest-client/ntaphttp"
	"github.com/greenqloud/ontap-rest-client/ntaptransport"

	"github.com/go-openapi/runtime"
	httptransport "github.com/go-openapi/runtime/client"
	"github.com/go-openapi/strfmt"
)

// http or https schemes
// TODO remove
const (
	HTTPScheme        = "http"
	HTTPSScheme       = "https"
	RetrySleepSec     = 1
	RequestMaxTimeSec = 15
	RetryMaxTrial     = 8
	NetAppAuth        = "X-Netapp-Authorization"
)

// DefaultHTTPRetryClient is http client with retries that is used by ontap rest client
// TODO: deprecate
var DefaultHTTPRetryClient *http.Client = NewDefaultHTTPRetryClient()

// NewOntapClient Construct a crawler for a ONTAP system
// Empty string of port treated as no port specified.
// Scheme is either Http or Https. Https is default by ONTAP. Http can be used for testing.
// Port is not needed to query ONTAP, but useful for testing infrastructure.
// This client does automatic retry on GET methods at transport layer
// See retry details in NewDefaultHTTPRetryClient function
// TODO: deprecate
func NewOntapClient(ipaddress string, port string, username string, password string, scheme string) *Ontap {
	return NewOntapClientWithHTTPClient(ipaddress, port, username, password, scheme, DefaultHTTPRetryClient)
}

// NewOntapClientWithConfig returns a ontap client based on configs
// This is intended to be used by RT. Client of this lib should specify the configs and no value should be hard coded
func NewOntapClientWithConfig(httpCfg *ntaphttp.Config, transportCfg *ntaptransport.Config) *Ontap {
	// ensure the default base path
	if transportCfg.BasePath == "" {
		transportCfg = transportCfg.WithBasePath(DefaultBasePath)
	}
	httpClient := ntaphttp.NewHTTPRetryClient(httpCfg)
	rt := ntaptransport.NewTransportWithClient(transportCfg, httpClient)
	return New(rt, strfmt.Default)
}

// NewOntapClientWithHTTPClient returns a ontap client using httpclient underneath
// Empty string of port treated as no port specified.
// Scheme is either Http or Https. Https is default by ONTAP. Http can be used for testing.
// Port is not needed to query ONTAP, but useful for testing infrastructure.
// TODO: deprecate
func NewOntapClientWithHTTPClient(ipaddress string, port string, username string, password string, scheme string, httpclient *http.Client) *Ontap {

	cfg := ntaptransport.NewConfig().
		WithIPAddress(ipaddress).
		WithPort(port).
		WithDefaultAuth(httptransport.BasicAuth(username, password)).
		WithSchemes([]ntaptransport.Scheme{ntaptransport.Scheme(scheme)}).
		WithDefaultMediaType("application/json").
		WithBasePath(DefaultBasePath)

	return NewOntapClientWithHTTPClientAndTransportConfig(cfg, httpclient)
}

// NewOntapClientWithHTTPClientAndBearerToken returns http ontap client that supports bearer token
func NewOntapClientWithHTTPClientAndBearerToken(ipaddress string, port string, username string, password string, scheme string, token string, httpclient *http.Client) *Ontap {
	cfg := ntaptransport.NewConfig().
		WithIPAddress(ipaddress).
		WithPort(port).
		WithDefaultAuth(ProcessAndCombineAuths(httptransport.BasicAuth(username, password), httptransport.APIKeyAuth(NetAppAuth, "header", token))).
		WithSchemes([]ntaptransport.Scheme{ntaptransport.Scheme(scheme)}).
		WithDefaultMediaType("application/json").
		WithBasePath(DefaultBasePath)

	return NewOntapClientWithHTTPClientAndTransportConfig(cfg, httpclient)
}

// ProcessAndCombineAuths process and combines multiple auths when sent in request
func ProcessAndCombineAuths(auths ...runtime.ClientAuthInfoWriter) runtime.ClientAuthInfoWriter {
	return runtime.ClientAuthInfoWriterFunc(func(r runtime.ClientRequest, _ strfmt.Registry) error {
		for _, auth := range auths {
			if auth == nil {
				continue
			}
			err := auth.AuthenticateRequest(r, nil)
			if err != nil {
				return err
			}
		}
		return nil
	})
}

// NewOntapClientWithHTTPClientAndTransportConfig returns an ontap client based on the transport config
// and use the httpclient
func NewOntapClientWithHTTPClientAndTransportConfig(cfg *ntaptransport.Config, httpclient *http.Client) *Ontap {
	rt := ntaptransport.NewTransportWithClient(cfg, httpclient)
	return New(rt, strfmt.Default)
}

// NewDefaultHTTPRetryClient constructs and returns a new
// http.Client with retry in transport layer which is used in default NewOntapClient function
//
// It skips SSL verify.
//
// Retries 8 times for every GET request on
// * Temporary transport error
// * HTTP status codes: 408, 500, 503, 504
//
// HTTP timeout is 15 sec per request
//
// Assume requests GET is idempotent (does not change server state when issued multiple times).
// For non-idempotent requests (POST,PATCH,DELETE), user of this library at application layer needs to first
// check with server (GET) if records exists or not, and then retry.
// TODO: deprecate
func NewDefaultHTTPRetryClient() *http.Client {
	// create the transport to disable SSL verification
	// this transport implements RoundTripper interface
	cfg := ntaphttp.NewConfig().
		WithDelayBaseSec(RetrySleepSec).
		WithRetryDelayType(ntaphttp.ConstantDelay).
		WithInsecureSkipVerify(true).
		WithRequestTimeMaxSec(RequestMaxTimeSec).
		WithRetryHTTPCodes(ntaphttp.RetryableHTTPCodes).
		WithRetryHTTPMethods([]ntaphttp.HTTPMethod{ntaphttp.GET}).
		WithRetryMaxTrial(RetryMaxTrial).
		WithLogger(logrus.New())

	httpclient := ntaphttp.NewHTTPRetryClient(cfg)

	return httpclient
}
