// Code generated by go-swagger; DO NOT EDIT.

package ntapmodel

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"encoding/json"

	strfmt "github.com/go-openapi/strfmt"

	"github.com/go-openapi/errors"
	"github.com/go-openapi/swag"
	"github.com/go-openapi/validate"
)

// CloudTargetReference cloud target
// swagger:model cloudTargetReference
type CloudTargetReference struct {

	// links
	Links *CloudTargetReferenceLinks `json:"_links,omitempty"`

	// Access key ID for AWS_S3 and other S3 compatible provider types.
	AccessKey string `json:"access_key,omitempty"`

	// Authentication used to access the target. SnapMirror does not yet support CAP. Required in POST.
	// Enum: [key cap ec2_iam gcp_sa azure_msi]
	AuthenticationType string `json:"authentication_type,omitempty"`

	// Azure account
	AzureAccount string `json:"azure_account,omitempty"`

	// Azure access key
	AzurePrivateKey string `json:"azure_private_key,omitempty"`

	// This parameter is available only when auth-type is CAP.
	// It specifies a full URL of the request to a CAP server for retrieving temporary credentials (access-key, secret-pasword, and session token) for accessing the object store.
	//
	CapURL string `json:"cap_url,omitempty"`

	// Is SSL/TLS certificate validation enabled? The default value is true.
	// This can only be modified for SGWS, IBM_COS, and ONTAP_S3 provider types.
	//
	CertificateValidationEnabled bool `json:"certificate_validation_enabled,omitempty"`

	// cluster
	Cluster *ClusterReference `json:"cluster,omitempty"`

	// Data bucket/container name
	Container string `json:"container,omitempty"`

	// ipspace
	Ipspace *CloudTargetReferenceIpspace `json:"ipspace,omitempty"`

	// cloud target name
	Name string `json:"name,omitempty"`

	// Owner of the target. Allowed values are FabricPool or SnapMirror. A target can be used by only one feature.
	// Enum: [fabricpool snapmirror]
	Owner string `json:"owner,omitempty"`

	// Port number of the object store that ONTAP uses when establishing a connection. Required in POST.
	Port int64 `json:"port,omitempty"`

	// Type of cloud provider. Allowed values depend on owner type.
	// For FabricPool, AliCloud, AWS_S3, Azure_Cloud, GoggleCloud, IBM_COS, SGWS, and ONTAP_S3 are allowed.
	// For SnapMirror, the valid values are AWS_S3 or SGWS
	//
	ProviderType string `json:"provider_type,omitempty"`

	// Secret access key for AWS_S3 and other S3 compatible provider types.
	SecretPassword string `json:"secret_password,omitempty"`

	// Fully qualified domain name of the object store server. Required on POST.
	// For Amazon S3, server name must be an AWS regional endpoint in the format s3.amazonaws.com or s3-.amazonaws.com, for example, s3-us-west-2.amazonaws.com.
	// The region of the server and the bucket must match.
	// For Azure, if the server is a “blob.core.windows.net” or a "blob.core.usgovcloudapi.net", then a value of azure-account followed by a period is added in front of the server.
	//
	Server string `json:"server,omitempty"`

	// Encryption of data at rest by the object store server for AWS_S3 and other S3 compatible provider types.
	// This is an advanced property. In most cases it is best not to change default value of “sse_s3” for object store servers which support SSE-S3 encryption.
	// The encryption is in addition to any encryption done by ONTAP at a volume or at an aggregate level.
	// Note that changing this option does not change encryption of data which already exist in the object store.
	//
	// Enum: [none sse_s3]
	ServerSideEncryption string `json:"server_side_encryption,omitempty"`

	// Use of the cloud target by SnapMirror.
	// Enum: [data metadata]
	SnapmirrorUse string `json:"snapmirror_use,omitempty"`

	// SSL/HTTPS enabled or not
	SslEnabled bool `json:"ssl_enabled,omitempty"`

	// svm
	Svm *SvmReference `json:"svm,omitempty"`

	// URL style used to access S3 bucket.
	// Enum: [path_style virtual_hosted_style]
	URLStyle string `json:"url_style,omitempty"`

	// Use HTTP proxy when connecting to the object store.
	UseHTTPProxy *bool `json:"use_http_proxy,omitempty"`

	// The amount of cloud space used by all the aggregates attached to the target, in bytes.
	// This field is only populated for FabricPool targets. The value is recalculated once every 5 minutes.
	//
	// Read Only: true
	Used int64 `json:"used,omitempty"`

	// cloud target UUID
	// Read Only: true
	// Format: uuid
	UUID strfmt.UUID `json:"uuid,omitempty"`
}

// Validate validates this cloud target reference
func (m *CloudTargetReference) Validate(formats strfmt.Registry) error {
	var res []error

	if err := m.validateLinks(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateAuthenticationType(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateCluster(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateIpspace(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateOwner(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateServerSideEncryption(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateSnapmirrorUse(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateSvm(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateURLStyle(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateUUID(formats); err != nil {
		res = append(res, err)
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

func (m *CloudTargetReference) validateLinks(formats strfmt.Registry) error {

	if swag.IsZero(m.Links) { // not required
		return nil
	}

	if m.Links != nil {
		if err := m.Links.Validate(formats); err != nil {
			if ve, ok := err.(*errors.Validation); ok {
				return ve.ValidateName("_links")
			}
			return err
		}
	}

	return nil
}

var cloudTargetReferenceTypeAuthenticationTypePropEnum []interface{}

func init() {
	var res []string
	if err := json.Unmarshal([]byte(`["key","cap","ec2_iam","gcp_sa","azure_msi"]`), &res); err != nil {
		panic(err)
	}
	for _, v := range res {
		cloudTargetReferenceTypeAuthenticationTypePropEnum = append(cloudTargetReferenceTypeAuthenticationTypePropEnum, v)
	}
}

const (

	// CloudTargetReferenceAuthenticationTypeKey captures enum value "key"
	CloudTargetReferenceAuthenticationTypeKey string = "key"

	// CloudTargetReferenceAuthenticationTypeCap captures enum value "cap"
	CloudTargetReferenceAuthenticationTypeCap string = "cap"

	// CloudTargetReferenceAuthenticationTypeEc2Iam captures enum value "ec2_iam"
	CloudTargetReferenceAuthenticationTypeEc2Iam string = "ec2_iam"

	// CloudTargetReferenceAuthenticationTypeGcpSa captures enum value "gcp_sa"
	CloudTargetReferenceAuthenticationTypeGcpSa string = "gcp_sa"

	// CloudTargetReferenceAuthenticationTypeAzureMsi captures enum value "azure_msi"
	CloudTargetReferenceAuthenticationTypeAzureMsi string = "azure_msi"
)

// prop value enum
func (m *CloudTargetReference) validateAuthenticationTypeEnum(path, location string, value string) error {
	if err := validate.Enum(path, location, value, cloudTargetReferenceTypeAuthenticationTypePropEnum); err != nil {
		return err
	}
	return nil
}

func (m *CloudTargetReference) validateAuthenticationType(formats strfmt.Registry) error {

	if swag.IsZero(m.AuthenticationType) { // not required
		return nil
	}

	// value enum
	if err := m.validateAuthenticationTypeEnum("authentication_type", "body", m.AuthenticationType); err != nil {
		return err
	}

	return nil
}

func (m *CloudTargetReference) validateCluster(formats strfmt.Registry) error {

	if swag.IsZero(m.Cluster) { // not required
		return nil
	}

	if m.Cluster != nil {
		if err := m.Cluster.Validate(formats); err != nil {
			if ve, ok := err.(*errors.Validation); ok {
				return ve.ValidateName("cluster")
			}
			return err
		}
	}

	return nil
}

func (m *CloudTargetReference) validateIpspace(formats strfmt.Registry) error {

	if swag.IsZero(m.Ipspace) { // not required
		return nil
	}

	if m.Ipspace != nil {
		if err := m.Ipspace.Validate(formats); err != nil {
			if ve, ok := err.(*errors.Validation); ok {
				return ve.ValidateName("ipspace")
			}
			return err
		}
	}

	return nil
}

var cloudTargetReferenceTypeOwnerPropEnum []interface{}

func init() {
	var res []string
	if err := json.Unmarshal([]byte(`["fabricpool","snapmirror"]`), &res); err != nil {
		panic(err)
	}
	for _, v := range res {
		cloudTargetReferenceTypeOwnerPropEnum = append(cloudTargetReferenceTypeOwnerPropEnum, v)
	}
}

const (

	// CloudTargetReferenceOwnerFabricpool captures enum value "fabricpool"
	CloudTargetReferenceOwnerFabricpool string = "fabricpool"

	// CloudTargetReferenceOwnerSnapmirror captures enum value "snapmirror"
	CloudTargetReferenceOwnerSnapmirror string = "snapmirror"
)

// prop value enum
func (m *CloudTargetReference) validateOwnerEnum(path, location string, value string) error {
	if err := validate.Enum(path, location, value, cloudTargetReferenceTypeOwnerPropEnum); err != nil {
		return err
	}
	return nil
}

func (m *CloudTargetReference) validateOwner(formats strfmt.Registry) error {

	if swag.IsZero(m.Owner) { // not required
		return nil
	}

	// value enum
	if err := m.validateOwnerEnum("owner", "body", m.Owner); err != nil {
		return err
	}

	return nil
}

var cloudTargetReferenceTypeServerSideEncryptionPropEnum []interface{}

func init() {
	var res []string
	if err := json.Unmarshal([]byte(`["none","sse_s3"]`), &res); err != nil {
		panic(err)
	}
	for _, v := range res {
		cloudTargetReferenceTypeServerSideEncryptionPropEnum = append(cloudTargetReferenceTypeServerSideEncryptionPropEnum, v)
	}
}

const (

	// CloudTargetReferenceServerSideEncryptionNone captures enum value "none"
	CloudTargetReferenceServerSideEncryptionNone string = "none"

	// CloudTargetReferenceServerSideEncryptionSseS3 captures enum value "sse_s3"
	CloudTargetReferenceServerSideEncryptionSseS3 string = "sse_s3"
)

// prop value enum
func (m *CloudTargetReference) validateServerSideEncryptionEnum(path, location string, value string) error {
	if err := validate.Enum(path, location, value, cloudTargetReferenceTypeServerSideEncryptionPropEnum); err != nil {
		return err
	}
	return nil
}

func (m *CloudTargetReference) validateServerSideEncryption(formats strfmt.Registry) error {

	if swag.IsZero(m.ServerSideEncryption) { // not required
		return nil
	}

	// value enum
	if err := m.validateServerSideEncryptionEnum("server_side_encryption", "body", m.ServerSideEncryption); err != nil {
		return err
	}

	return nil
}

var cloudTargetReferenceTypeSnapmirrorUsePropEnum []interface{}

func init() {
	var res []string
	if err := json.Unmarshal([]byte(`["data","metadata"]`), &res); err != nil {
		panic(err)
	}
	for _, v := range res {
		cloudTargetReferenceTypeSnapmirrorUsePropEnum = append(cloudTargetReferenceTypeSnapmirrorUsePropEnum, v)
	}
}

const (

	// CloudTargetReferenceSnapmirrorUseData captures enum value "data"
	CloudTargetReferenceSnapmirrorUseData string = "data"

	// CloudTargetReferenceSnapmirrorUseMetadata captures enum value "metadata"
	CloudTargetReferenceSnapmirrorUseMetadata string = "metadata"
)

// prop value enum
func (m *CloudTargetReference) validateSnapmirrorUseEnum(path, location string, value string) error {
	if err := validate.Enum(path, location, value, cloudTargetReferenceTypeSnapmirrorUsePropEnum); err != nil {
		return err
	}
	return nil
}

func (m *CloudTargetReference) validateSnapmirrorUse(formats strfmt.Registry) error {

	if swag.IsZero(m.SnapmirrorUse) { // not required
		return nil
	}

	// value enum
	if err := m.validateSnapmirrorUseEnum("snapmirror_use", "body", m.SnapmirrorUse); err != nil {
		return err
	}

	return nil
}

func (m *CloudTargetReference) validateSvm(formats strfmt.Registry) error {

	if swag.IsZero(m.Svm) { // not required
		return nil
	}

	if m.Svm != nil {
		if err := m.Svm.Validate(formats); err != nil {
			if ve, ok := err.(*errors.Validation); ok {
				return ve.ValidateName("svm")
			}
			return err
		}
	}

	return nil
}

var cloudTargetReferenceTypeURLStylePropEnum []interface{}

func init() {
	var res []string
	if err := json.Unmarshal([]byte(`["path_style","virtual_hosted_style"]`), &res); err != nil {
		panic(err)
	}
	for _, v := range res {
		cloudTargetReferenceTypeURLStylePropEnum = append(cloudTargetReferenceTypeURLStylePropEnum, v)
	}
}

const (

	// CloudTargetReferenceURLStylePathStyle captures enum value "path_style"
	CloudTargetReferenceURLStylePathStyle string = "path_style"

	// CloudTargetReferenceURLStyleVirtualHostedStyle captures enum value "virtual_hosted_style"
	CloudTargetReferenceURLStyleVirtualHostedStyle string = "virtual_hosted_style"
)

// prop value enum
func (m *CloudTargetReference) validateURLStyleEnum(path, location string, value string) error {
	if err := validate.Enum(path, location, value, cloudTargetReferenceTypeURLStylePropEnum); err != nil {
		return err
	}
	return nil
}

func (m *CloudTargetReference) validateURLStyle(formats strfmt.Registry) error {

	if swag.IsZero(m.URLStyle) { // not required
		return nil
	}

	// value enum
	if err := m.validateURLStyleEnum("url_style", "body", m.URLStyle); err != nil {
		return err
	}

	return nil
}

func (m *CloudTargetReference) validateUUID(formats strfmt.Registry) error {

	if swag.IsZero(m.UUID) { // not required
		return nil
	}

	if err := validate.FormatOf("uuid", "body", "uuid", m.UUID.String(), formats); err != nil {
		return err
	}

	return nil
}

// MarshalBinary interface implementation
func (m *CloudTargetReference) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *CloudTargetReference) UnmarshalBinary(b []byte) error {
	var res CloudTargetReference
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}

// CloudTargetReferenceIpspace ipspace information
// swagger:model CloudTargetReferenceIpspace
type CloudTargetReferenceIpspace struct {

	// links
	Links *CloudTargetReferenceIpspaceLinks `json:"_links,omitempty"`

	// IPspace name
	Name string `json:"name,omitempty"`

	// IPspace UUID
	// Format: uuid
	UUID strfmt.UUID `json:"uuid,omitempty"`
}

// Validate validates this cloud target reference ipspace
func (m *CloudTargetReferenceIpspace) Validate(formats strfmt.Registry) error {
	var res []error

	if err := m.validateLinks(formats); err != nil {
		res = append(res, err)
	}

	if err := m.validateUUID(formats); err != nil {
		res = append(res, err)
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

func (m *CloudTargetReferenceIpspace) validateLinks(formats strfmt.Registry) error {

	if swag.IsZero(m.Links) { // not required
		return nil
	}

	if m.Links != nil {
		if err := m.Links.Validate(formats); err != nil {
			if ve, ok := err.(*errors.Validation); ok {
				return ve.ValidateName("ipspace" + "." + "_links")
			}
			return err
		}
	}

	return nil
}

func (m *CloudTargetReferenceIpspace) validateUUID(formats strfmt.Registry) error {

	if swag.IsZero(m.UUID) { // not required
		return nil
	}

	if err := validate.FormatOf("ipspace"+"."+"uuid", "body", "uuid", m.UUID.String(), formats); err != nil {
		return err
	}

	return nil
}

// MarshalBinary interface implementation
func (m *CloudTargetReferenceIpspace) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *CloudTargetReferenceIpspace) UnmarshalBinary(b []byte) error {
	var res CloudTargetReferenceIpspace
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}

// CloudTargetReferenceIpspaceLinks cloud target reference ipspace links
// swagger:model CloudTargetReferenceIpspaceLinks
type CloudTargetReferenceIpspaceLinks struct {

	// self
	Self *Href `json:"self,omitempty"`
}

// Validate validates this cloud target reference ipspace links
func (m *CloudTargetReferenceIpspaceLinks) Validate(formats strfmt.Registry) error {
	var res []error

	if err := m.validateSelf(formats); err != nil {
		res = append(res, err)
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

func (m *CloudTargetReferenceIpspaceLinks) validateSelf(formats strfmt.Registry) error {

	if swag.IsZero(m.Self) { // not required
		return nil
	}

	if m.Self != nil {
		if err := m.Self.Validate(formats); err != nil {
			if ve, ok := err.(*errors.Validation); ok {
				return ve.ValidateName("ipspace" + "." + "_links" + "." + "self")
			}
			return err
		}
	}

	return nil
}

// MarshalBinary interface implementation
func (m *CloudTargetReferenceIpspaceLinks) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *CloudTargetReferenceIpspaceLinks) UnmarshalBinary(b []byte) error {
	var res CloudTargetReferenceIpspaceLinks
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}

// CloudTargetReferenceLinks cloud target reference links
// swagger:model CloudTargetReferenceLinks
type CloudTargetReferenceLinks struct {

	// self
	Self *Href `json:"self,omitempty"`
}

// Validate validates this cloud target reference links
func (m *CloudTargetReferenceLinks) Validate(formats strfmt.Registry) error {
	var res []error

	if err := m.validateSelf(formats); err != nil {
		res = append(res, err)
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

func (m *CloudTargetReferenceLinks) validateSelf(formats strfmt.Registry) error {

	if swag.IsZero(m.Self) { // not required
		return nil
	}

	if m.Self != nil {
		if err := m.Self.Validate(formats); err != nil {
			if ve, ok := err.(*errors.Validation); ok {
				return ve.ValidateName("_links" + "." + "self")
			}
			return err
		}
	}

	return nil
}

// MarshalBinary interface implementation
func (m *CloudTargetReferenceLinks) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *CloudTargetReferenceLinks) UnmarshalBinary(b []byte) error {
	var res CloudTargetReferenceLinks
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
