package ntaptransport

import (
	"strconv"

	"github.com/go-openapi/runtime"
)

// Scheme is either http or https
type Scheme string

const (
	// HTTP is http scheme
	HTTP Scheme = "http"
	// HTTPS is https scheme
	HTTPS Scheme = "https"
)

// Config is config for transport
type Config struct {
	IPAddress        string   // required
	Port             string   // optional
	Schemes          []Scheme // empty then will pick http
	DefaultAuth      runtime.ClientAuthInfoWriter
	BasePath         string // required
	DefaultMediaType string // empty then will pick json
}

// NewConfig returns an empty config
func NewConfig() *Config {
	return &Config{}
}

// WithIPAddress sets IPAddress in config
// also accepts DNS name
func (c *Config) WithIPAddress(x string) *Config {
	c.IPAddress = x
	return c
}

// WithPort sets Port in config
func (c *Config) WithPort(x string) *Config {
	c.Port = x
	return c
}

// WithIntPort sets port in config
func (c *Config) WithIntPort(x int) *Config {
	c.Port = strconv.Itoa(x) // gives string base 10 representation of x
	return c
}

// WithSchemes sets Schemes in config
func (c *Config) WithSchemes(x []Scheme) *Config {
	c.Schemes = x
	return c
}

// WithDefaultAuth sets DefaultAuth in config
func (c *Config) WithDefaultAuth(x runtime.ClientAuthInfoWriter) *Config {
	c.DefaultAuth = x
	return c
}

// WithBasePath sets BasePath in config
func (c *Config) WithBasePath(x string) *Config {
	c.BasePath = x
	return c
}

// WithDefaultMediaType sets DefaultMediaType  in config
func (c *Config) WithDefaultMediaType(x string) *Config {
	c.DefaultMediaType = x
	return c
}
