package ntaptransport

import (
	"fmt"
	"net/http"

	"github.com/go-openapi/runtime"
	httptransport "github.com/go-openapi/runtime/client"
	"github.com/thoas/go-funk"
)

// NewTransportWithClient returns a new open api client transport
func NewTransportWithClient(cfg *Config, client *http.Client) runtime.ClientTransport {
	host := cfg.IPAddress
	if cfg.Port != "" {
		host = fmt.Sprintf("%s:%s", cfg.IPAddress, cfg.Port)
	}

	// Construct a runtime
	rt := httptransport.NewWithClient(host,
		cfg.BasePath,
		funk.Map(cfg.Schemes, func(x Scheme) string {
			return string(x)
		}).([]string), // If it is empty runtime will use http
		client)
	// add producer of hal json so the runtime treats it as json
	// Swagger by default does not recognize hal+json
	rt.Consumers["application/hal+json"] = runtime.JSONConsumer()
	rt.Producers["application/hal+json"] = runtime.JSONProducer()
	// add default auth here so that we do not need auth everytime
	rt.DefaultAuthentication = cfg.DefaultAuth
	// set json as default type to avoid getting href from ontap
	rt.DefaultMediaType = "application/json"
	if cfg.DefaultMediaType != "" {
		rt.DefaultMediaType = cfg.DefaultMediaType
	}
	return rt
}
