package qstackcrypto

import (
	"encoding/hex"
	"strconv"
)

const (
	pkcs7 padding = "PKCS7Padding"
	pkcs5 padding = "PKCS5Padding"

	pkcs7BlockSize = 16
	pkcs5BlockSize = 8
)

type padding string

type padder struct {
	blockSize int
}

func newPadder(version *Version) *padder {
	if version.Padding == string(pkcs7) {
		return &padder{blockSize: pkcs7BlockSize}
	}
	if version.Padding == string(pkcs5) {
		return &padder{blockSize: pkcs5BlockSize}
	}
	return nil
}

func (p *padder) pad(text []byte) []byte {
	c := p.blockSize - (len(text) % p.blockSize)
	for i := 0; i < c; i++ {
		text = append(text, byte(c))
	}

	return text
}

func (p *padder) trim(text []byte) (*string, error) {
	nl := len(text)
	x := hex.EncodeToString(text[nl-1:])
	c, err := strconv.ParseInt(x, 16, p.blockSize)
	if err != nil {
		return nil, err
	}

	trimmed := string(text[:nl-int(c)])
	return &trimmed, nil
}
